@@ -193,7 +193,7 @@ function(cross, chr, pheno.col=1, model=c("normal","binary","2part","np"),
     }
 
     # get genotype names
-    gen.names <- getgenonames(type,chrtype,"full",sexpgm)
+    gen.names <- getgenonames(type,chrtype,"full",sexpgm,attributes(cross))
     n.gen <- length(gen.names)
 
     # starting values for interval mapping
@@ -219,7 +219,8 @@ function(cross, chr, pheno.col=1, model=c("normal","binary","2part","np"),
 
       # revise X chromosome genotypes
       if(chrtype=="X" && (type=="bc" || type=="f2"))
-         newgeno <- reviseXdata(type, "full", sexpgm, geno=newgeno)
+         newgeno <- reviseXdata(type, "full", sexpgm, geno=newgeno,
+                                cross.attr=attributes(cross))
 
       n.pos <- ncol(newgeno)
       map <- cross$geno[[i]]$map
@@ -242,11 +243,21 @@ function(cross, chr, pheno.col=1, model=c("normal","binary","2part","np"),
 
       # revise X chromosome genotypes
       if(chrtype=="X" && (type=="bc" || type=="f2"))
-         draws <- reviseXdata(type, "full", sexpgm, draws=draws)
+         draws <- reviseXdata(type, "full", sexpgm, draws=draws,
+                              cross.attr=attributes(cross))
+
+      if("map" %in% names(attributes(cross$geno[[i]]$draws)))
+        map <- attr(cross$geno[[i]]$draws,"map")
+      else {
+        stp <- attr(cross$geno[[i]]$draws, "step")
+        oe <- attr(cross$geno[[i]]$draws, "off.end")
+      
+        if("stepwidth" %in% names(attributes(cross$geno[[i]]$draws)))
+          stpw <- attr(cross$geno[[i]]$draws, "stepwidth")
+        else stpw <- "fixed"
+        map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
+      }
 
-      map <- create.map(cross$geno[[i]]$map,
-                        attr(cross$geno[[i]]$draws,"step"),
-                        attr(cross$geno[[i]]$draws,"off.end"))
       if(is.matrix(map)) {
         marnam <- colnames(map)
         map <- map[1,]
@@ -264,11 +275,21 @@ function(cross, chr, pheno.col=1, model=c("normal","binary","2part","np"),
 
       # revise X chromosome genotypes
       if(chrtype=="X" && (type=="bc" || type=="f2"))
-         genoprob <- reviseXdata(type, "full", sexpgm, prob=genoprob)
+         genoprob <- reviseXdata(type, "full", sexpgm, prob=genoprob,
+                                 cross.attr=attributes(cross))
+
+      if("map" %in% names(attributes(cross$geno[[i]]$prob)))
+        map <- attr(cross$geno[[i]]$prob,"map")
+      else {
+        stp <- attr(cross$geno[[i]]$prob, "step")
+        oe <- attr(cross$geno[[i]]$prob, "off.end")
+        
+        if("stepwidth" %in% names(attributes(cross$geno[[i]]$prob)))
+          stpw <- attr(cross$geno[[i]]$prob, "stepwidth")
+        else stpw <- "fixed"
+        map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
+      }
 
-      map <- create.map(cross$geno[[i]]$map,
-                        attr(cross$geno[[i]]$prob,"step"),
-                        attr(cross$geno[[i]]$prob,"off.end"))
       if(is.matrix(map)) {
         marnam <- colnames(map)
         map <- map[1,]
