@@ -2,8 +2,8 @@
 #
 # sim.geno.R
 #
-# copyright (c) 2001-5, Karl W Broman, Johns Hopkins University
-# last modified Oct, 2005
+# copyright (c) 2001-6, Karl W Broman, Johns Hopkins University
+# last modified Oct, 2006
 # first written Feb, 2001
 # Licensed under the GNU General Public License version 2 (June, 1991)
 # 
@@ -20,7 +20,8 @@
 
 sim.geno <-
 function(cross, n.draws=16, step=0, off.end=0, error.prob=0.0001,
-         map.function=c("haldane","kosambi","c-f","morgan"))
+         map.function=c("haldane","kosambi","c-f","morgan"),
+         stepwidth=c("fixed", "variable"))
 {
   # map function
   map.function <- match.arg(map.function)
@@ -29,6 +30,8 @@ function(cross, n.draws=16, step=0, off.end=0, error.prob=0.0001,
   else if(map.function=="morgan") mf <- mf.m
   else mf <- mf.h
 
+  stepwidth <- match.arg(stepwidth)
+
   # don't let error.prob be exactly zero, just in case
   if(error.prob < 1e-50) error.prob <- 1e-50
   if(error.prob > 1) {
@@ -78,7 +81,7 @@ function(cross, n.draws=16, step=0, off.end=0, error.prob=0.0001,
     # recombination fractions
     if(one.map) {
       # recombination fractions
-      map <- create.map(cross$geno[[i]]$map,step,temp.offend)
+      map <- create.map(cross$geno[[i]]$map,step,temp.offend,stepwidth)
       rf <- mf(diff(map))
       if(type=="risib" || type=="riself")
         rf <- adjust.rf.ri(rf,substr(type,3,nchar(type)),class(cross$geno[[i]]))
@@ -92,7 +95,7 @@ function(cross, n.draws=16, step=0, off.end=0, error.prob=0.0001,
       n.pos <- ncol(newgen)
     }
     else {
-      map <- create.map(cross$geno[[i]]$map,step,temp.offend)
+      map <- create.map(cross$geno[[i]]$map,step,temp.offend,stepwidth)
       rf <- mf(diff(map[1,]))
       rf[rf < 1e-14] <- 1e-14
       rf2 <- mf(diff(map[1,]))
@@ -141,10 +144,12 @@ function(cross, n.draws=16, step=0, off.end=0, error.prob=0.0001,
 
     # attribute set to the error.prob value used, for later
     #     reference
+    attr(cross$geno[[i]]$draws, "map") <- map
     attr(cross$geno[[i]]$draws,"error.prob") <- error.prob
     attr(cross$geno[[i]]$draws,"step") <- step
     attr(cross$geno[[i]]$draws,"off.end") <- temp.offend
     attr(cross$geno[[i]]$draws,"map.function") <- map.function
+    attr(cross$geno[[i]]$draws,"stepwidth") <- stepwidth
   }
 
   # store simulated genotypes as integers
